@@ -1,11 +1,11 @@
-/* $Id: Client.h,v 1.36 2003/07/07 08:57:12 bluehal Exp $ */
+/* $Id: Client.h,v 1.39 2004/03/28 00:28:58 bluehal Exp $ */
 /* Author : Scott Holden ( scotth@thezone.net )
    Modified : Yong-iL Joh ( tolkien@mizi.com )
    Modified : Jorge Garca ( Jorge.Garcia@uv.es )
  *
  * Email Checker Pop3/Imap4/Licq/Gicu/mbox/maildir/finger
  *
- * Last Updated : $Date: 2003/07/07 08:57:12 $
+ * Last Updated : $Date: 2004/03/28 00:28:58 $
  *
  */
 
@@ -28,15 +28,25 @@
 typedef unsigned int off_t;
 #endif
 
+#define BUF_BIG 256
+#define BUF_SMALL 32
+#define BUF_SIZE 1024
+
 struct msglst;
 typedef struct _mbox_t *Pop3;
 typedef struct _mbox_t {
-	char label[32];				/* Printed at left; max 5 chars */
-	char path[256];				/* Path to mailbox */
-	char notify[256];			/* Program to notify mail arrivation */
-	char action[256];			/* Action to execute on mouse click, reduces to what happens on button1 */
-	char button2[256];			/* What to run on button2. (middle) */
-	char fetchcmd[256];			/* Action for mail fetching for pop3/imap, reduces to what happens on button3 */
+	char label[BUF_SMALL];		/* Printed at left; max 5 chars */
+	char path[BUF_BIG];			/* Path to mailbox */
+	char notify[BUF_BIG];		/* Program to notify mail arrivation */
+	char action[BUF_BIG];		/* Action to execute on mouse click, reduces to
+								 *  what happens on button1. this is executed after
+								 *  either actionnew or actionnonew (if they are
+								 *  defined in the config file) */
+	char actionnew[BUF_BIG];	/* Action to execute on mouse click when new mail */
+	char actionnonew[BUF_BIG];	/* Action to execute on mouse click when no new mail */
+	char actiondc[BUF_BIG];		/* Action to execute when icq is disconnected */
+	char button2[BUF_BIG];		/* What to run on button2. (middle) */
+	char fetchcmd[BUF_BIG];		/* Action for mail fetching for pop3/imap, reduces to what happens on button3 */
 	int fetchinterval;
 	int TotalMsgs;				/* Total messages in mailbox */
 	int UnreadMsgs;				/* New (unread) messages in mailbox */
@@ -65,15 +75,18 @@ typedef struct _mbox_t {
 			unsigned int dircache_flush:1;	/* hack to flush directory caches */
 		} maildir;
 		struct {
-			char password[32];
-			char userName[32];
-			char serverName[256];
+			char password[BUF_SMALL];
+			char userName[BUF_SMALL];
+			char serverName[BUF_BIG];
 			int serverPort;
 			int localPort;
 			char authList[100];
 			unsigned int dossl:1;	/* use tls. */
 			/* prompt the user if we can't login / password is empty */
 			unsigned int interactive_password:1;
+			/* using the msglst feature, fetch the headers
+			   to have them on hand */
+			unsigned int wantCacheHeaders:1;
 			unsigned char password_len;	/* memfrob may shorten passwords */
 		} pop_imap;
 	} u;
@@ -93,8 +106,6 @@ typedef struct _mbox_t {
 	const char *askpass;
 } mbox_t;
 
-#define BUF_SIZE 1024
-
 /* creation calls must have this prototype */
 int pop3Create( /*@notnull@ */ Pop3 pc, const char *str);
 int imap4Create( /*@notnull@ */ Pop3 pc, const char *str);
@@ -153,6 +164,12 @@ do { \
 #endif
 #endif							/* gnuc */
 #endif							/* client.h */
+
+#ifndef FALSE
+#define FALSE (0)
+#define TRUE (!FALSE)
+#endif
+
 /* vim:set ts=4: */
 /*
  * Local Variables:
