@@ -1,4 +1,4 @@
-/* $Id: charutil.h,v 1.5 2002/02/02 18:04:19 jordi Exp $ */
+/* $Id: charutil.h,v 1.11 2003/01/28 11:12:43 bluehal Exp $ */
 /* Author: Mark Hurley  (debian4tux@telocity.com)
  *
  * Character / string manipulation utilities. 
@@ -8,13 +8,15 @@
 #ifndef CHARUTIL
 #define CHARUTIL
 
-#ifdef EXT_GNU_REGEX_LIB
-#include <gnuregex.h>
-#else
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef HAVE_REGEX_H
 #include <regex.h>
 #endif
 
-int FullTrim(char *psValue);
+void FullTrim(char *psValue);
 
 void Bin2Hex(unsigned char *src, int length, char *dst);
 
@@ -28,7 +30,15 @@ void copy_substring(char *destination,
 /* common to Pop3 and Imap4 authentication list grabber. */
 void grab_authList(const char *source, char *destination);
 
+#ifdef USE_GNU_REGEX
 /* handles main regex work */
 int compile_and_match_regex(const char *regex, const char *str,
 							/*@out@ */ struct re_registers *regs);
 #endif
+
+/* acts like perl's function of the same name */
+void chomp(char *s);
+
+/* same as xstrdup, just better named ;) */
+char *strdup_ordie(const char *c);
+#endif
