@@ -55,5 +55,5 @@ int tlscomm_expect(struct connection_state *scs, const char *prefix,
 void tlscomm_close( /*@only@ */ struct connection_state *scs);
 
 /* internal function exported for testing */
-int
-getline_from_buffer(char *readbuffer, char *linebuffer, int linebuflen);
+int getline_from_buffer(char *readbuffer, char *linebuffer,
+						int linebuflen);
